#ifndef MPIPE_API_H
#define MPIPE_API_H

#include "mpipe_types.h"
#include "mpipe_log.h"
#include <stddef.h>

// версии api подконтрольные программисту, меняются осознанно руками

#define MPIPE_API_MAJOR_VERSION 1
#define MPIPE_API_MINOR_VERSION 0

EXTERN_C_BEGIN

// информация о библиотеке
typedef struct mpipe_info_t
{
    const char*         name;                           // имя библиотеки
    const char*         description;                    // описание библиотеки
    size_t              major_version_number;           // мажорная версия библиотеки
    size_t              minor_version_number;           // минорная версия библиотеки
    size_t              build_number;                   // номер сборки
    const char*         build_time;                     // время сборки (в формате UTC)
    size_t              api_major_version_number;       // мажорная версия библиотеки
    size_t              api_minor_version_number;       // минорная версия библиотеки
    const char*         host_platform;                  // платформа

} mpipe_info_t;

// параметры движка
typedef struct mpipe_engine_params_t
{
    size_t              max_workers; // 0 - auto
    mpipe_log_level_t   log_level;
    mpipe_cb_log_t      on_log;

} mpipe_engine_params_t;

EXPORT_API enum mpipe_result_t mpipe_engine_init(const struct mpipe_engine_params_t* params);
EXPORT_API enum mpipe_result_t mpipe_engine_cleanup();
EXPORT_API const mpipe_info_t* mpipe_info();

EXTERN_C_END

#endif // MPIPE_API_H
