#ifndef MPIPE_API_CONVERTER_H
#define MPIPE_API_CONVERTER_H

#include "mpipe_media_frame.h"

EXTERN_C_BEGIN

// описатель медиа-конвертора
typedef struct mpipe_converter_t
{
    mpipe_handle_t              handle;     // номер дескриптора
    mpipe_media_format_t        format;     // выходной формат
    void*                       user_data;  // пользовательский контекст

} mpipe_converter_t;

typedef void (*mpipe_cb_converter_frame_t)(const struct mpipe_converter_t* converter
                                          , const struct mpipe_media_frame_t* media_frame);

// блок описания функции обратного вызова для конвертера
typedef struct mpipe_converter_cb_t
{
    mpipe_cb_converter_frame_t  on_frame;
} mpipe_converter_cb_t;

// параметры конвертера
typedef struct mpipe_converter_params_t
{
    enum mpipe_converter_type_t     converter_type;     // тип конверетра
    const char*                     options;            // специальные параметры (зависят от кодека)
    mpipe_media_format_t            format;             // выходной (для декодера входной) формат потока
    void*                           user_data;          // пользовательские данные
    mpipe_converter_cb_t            cb;                 // блок обратного вызова

} mpipe_converter_params_t;

// создать конвертер
EXPORT_API mpipe_handle_t mpipe_converter_create(const struct mpipe_converter_params_t* params);

// отправить медиафрейм в ковертер для конвертирования
EXPORT_API enum mpipe_result_t mpipe_converter_send_frame(mpipe_handle_t handle
                                                         , const struct mpipe_media_frame_t* media_frame);


EXTERN_C_END

#endif // MPIPE_API_CONVERTER_H
