#ifndef MPIPE_API_MEDIA_FORMAT_H
#define MPIPE_API_MEDIA_FORMAT_H

#include "mpipe_types.h"
#include "mpipe_common.h"

EXTERN_C_BEGIN

// медиапараметры зависящие от типа кодека или формата данных
typedef struct mpipe_media_params_t
{
    const char*                     name;
    size_t                          bitrate;        // битрейт потока (только для энкодера)
    size_t                          gop;            // длина группы ключевых кдров (для видео-энкодеров)
    size_t                          frame_size;     // размер фрейма в семлах (только для audio)
    uint32_t                        profile_id;     // профиль и
    uint32_t                        level_id;       // уровень энкодера
    mpipe_data_t                    extra_data;     // специальный блок конфигурации кодека (зависит от типа кодека)
    const char*                     options;        // специальные опции для кодека (зависит от типа)
    uint32_t                        tag;            // экспериментальное поле для разработчика библиотеки

} media_params_t;

// описатель аудио-формата
typedef struct mpipe_audio_info_t
{
    enum mpipe_audio_format_id_t    format_id;      // ID-формата
    uint32_t                        sample_rate;    // частота дескритизации
    uint32_t                        channels;       // количество каналов

} mpipe_audio_info_t;

// описатель видео-формата
typedef struct mpipe_video_info_t
{
    enum mpipe_video_format_id_t    format_id;      // ID-формата
    int32_t                         width;          // ширина кадра
    int32_t                         height;         // высота кадра
    float                           frame_rate;     // частота кадров (в сек)

} mpipe_video_info_t;

// обобщеный описатель медиаформата
typedef struct mpipe_media_info_t
{
    enum mpipe_media_type_t         media_type;     // тип медиа
    union
    {
        struct mpipe_audio_info_t   audio;
        struct mpipe_video_info_t   video;
    }                               info;           // блок описания зависимый от типа

} mpipe_media_info_t;

// обобщеная структура описания медиа формата
typedef struct mpipe_media_format_t
{
    struct mpipe_media_info_t       media_info;
    struct mpipe_media_params_t     media_params;

} mpipe_media_format_t;


EXTERN_C_END

#endif // MPIPE_API_MEDIA_FORMAT_H
