#ifndef MPIPE_API_STREAM_H
#define MPIPE_API_STREAM_H

#include "mpipe_media_format.h"

EXTERN_C_BEGIN

struct mpipe_media_frame_t;

// обобщенный идентификатор потока
typedef struct mpipe_media_stream_info_t
{
    mpipe_stream_id_t               stream_id;          // номер потока
    enum mpipe_stream_direction_t   direction;          // направление потока

} mpipe_media_stream_info_t;

// описатель потока
typedef struct mpipe_media_stream_t
{
    mpipe_media_stream_info_t       info;               // идентификатор
    const char*                     name;               // строковое имя потока (необязательное)
    mpipe_media_format_t            format;             // формат потока
    mpipe_bool_t                    enabled;            // состояние: включен/отключен

} mpipe_media_stream_t;

typedef struct mpipe_stream_metrics_t
{
    size_t     id;                                      // идентификатор статистики (увеличивается на 1 при каждом подсчете)
    size_t     total_frames;                            // полное количество фреймов с начала сеанса
    size_t     total_bytes;                             // полное количество байт с начала сейнса
    double     framerate;                               // среднее количество фреймов в секунду
    size_t     bitrate;                                 // средний битрейт

} mpipe_stream_metrics_t;

typedef void (*mpipe_cb_stream_frame_t)(mpipe_handle_t object_handle
                                        , const struct mpipe_media_stream_t* media_stream
                                        , const struct mpipe_media_frame_t* media_frame);

typedef void (*mpipe_cb_stream_info_t)(const struct mpipe_media_stream_t* media_stream
                                        , const struct mpipe_stream_metrics_t* stream_metrics);

// получить количество стримов объекта для конкретного направления
EXPORT_API int32_t mpipe_stream_get_count(mpipe_handle_t object_handle
                                          , mpipe_stream_direction_t direction);

// отправить фрейм в поток объекта
EXPORT_API enum mpipe_result_t mpipe_stream_send_frame(mpipe_handle_t object_handle
                                                        , const struct mpipe_media_stream_info_t* stream_info
                                                        , const struct mpipe_media_frame_t* media_frame);

// получить состояние (описатель) стрима по дескритору объекта
EXPORT_API enum mpipe_result_t mpipe_stream_get_state(mpipe_handle_t object_handle
                                                        , const struct mpipe_media_stream_info_t* stream_info
                                                        , struct mpipe_media_stream_t* media_stream);

// получить метрики стрима по дескриптору объекта
EXPORT_API enum mpipe_result_t mpipe_stream_get_metrics(mpipe_handle_t object_handle
                                                        , const struct mpipe_media_stream_info_t* stream_info
                                                        , struct mpipe_stream_metrics_t* stream_metrics);

// получить всю информацию по стримам для объекта по его дескриптору
EXPORT_API enum mpipe_result_t mpipe_stream_get_all_info(mpipe_handle_t object_handle
                                                        , mpipe_cb_stream_info_t on_stream_info);

EXTERN_C_END

#endif // MPIPE_API_STREAM_H
