#ifndef MPIPE_CONST_BUFFER_IMPL_H
#define MPIPE_CONST_BUFFER_IMPL_H

#include "i_buffer.h"

namespace mpipe
{

class const_buffer_basic : public i_buffer
{
public:
    using u_ptr_t = std::unique_ptr<const_buffer_basic>;

    // i_buffer interface
public:
    buffer_type_t buffer_type() const final;

};

class const_buffer_ref : public const_buffer_basic
{
    const void*     m_data_ref;
    std::size_t     m_size_ref;
public:

    using u_ptr_t = std::unique_ptr<const_buffer_ref>;

    static u_ptr_t create(const void* data_ref = nullptr
                           , std::size_t size_ref = 0);

    const_buffer_ref(const void* data_ref = nullptr
                          , std::size_t size_ref = 0);

    void assign(const void* data_ref
                , std::size_t size_ref);

    // i_data_object interface
public:
    const void *data() const;
    std::size_t size() const;

    // i_buffer interface
public:
    buffer_data_type_t data_type() const;
    std::size_t refs() const;
    i_buffer::u_ptr_t clone() const;
};

class const_buffer_shared : public const_buffer_basic
{
    i_data_object::s_ptr_t  m_shared_data;

public:
    using u_ptr_t = std::unique_ptr<const_buffer_shared>;

    static u_ptr_t create(i_data_object::s_ptr_t shared_data = nullptr);

    const_buffer_shared(i_data_object::s_ptr_t shared_data = nullptr);

    void assign(i_data_object::s_ptr_t shared_data);
    i_data_object::s_ptr_t shared_data() const;
    const_buffer_shared fork() const;

    // i_data_object interface
public:
    const void *data() const;
    std::size_t size() const;

    // i_buffer interface
public:
    buffer_data_type_t data_type() const;
    std::size_t refs() const;
    i_buffer::u_ptr_t clone() const;
};


}

#endif // MPIPE_CONST_BUFFER_IMPL_H
