#ifndef MPIPE_MAPPED_BUFFER_IMPL_H
#define MPIPE_MAPPED_BUFFER_IMPL_H

#include "i_mapped_buffer.h"
#include <vector>
#include <cstdint>

namespace mpipe
{

class mapped_buffer_basic : public i_mapped_buffer
{
public:
    using stored_data_t = std::vector<std::uint8_t>;
    using u_ptr_t = std::unique_ptr<mapped_buffer_basic>;

    // i_buffer interface
public:
    buffer_type_t buffer_type() const;
};

class mapped_buffer_ref : public mapped_buffer_basic
{
    void*           m_data_ref;
    std::size_t     m_size_ref;
public:

    using u_ptr_t = std::unique_ptr<mapped_buffer_ref>;

    static u_ptr_t create(void* data_ref = nullptr
                           , std::size_t size_ref = 0);

    mapped_buffer_ref(void* data_ref = nullptr
                          , std::size_t size_ref = 0);

    void assign(void* data_ref
                , std::size_t size_ref);

    // i_data_object interface
public:
    const void *data() const;
    std::size_t size() const;

    // i_buffer interface
public:
    buffer_data_type_t data_type() const;
    std::size_t refs() const;
    i_buffer::u_ptr_t clone() const;

    // i_mapped_buffer interface
public:
    void *map();
};

class mapped_buffer_stored : public mapped_buffer_basic
{
    stored_data_t       m_stored_data;
public:

    using u_ptr_t = std::unique_ptr<mapped_buffer_stored>;

    static u_ptr_t create(stored_data_t&& stored_data = {});

    mapped_buffer_stored(stored_data_t&& stored_data = {});

    void assign(stored_data_t&& stored_data);
    stored_data_t release();

    // i_data_object interface
public:
    const void *data() const;
    std::size_t size() const;

    // i_buffer interface
public:
    buffer_data_type_t data_type() const;
    std::size_t refs() const;
    i_buffer::u_ptr_t clone() const;

    // i_mapped_buffer interface
public:
    void *map();
};



}

#endif // MPIPE_MAPPED_BUFFER_IMPL_H
