#ifndef MPIPE_MUTABLE_BUFFER_IMPL_H
#define MPIPE_MUTABLE_BUFFER_IMPL_H

#include "i_mutable_buffer.h"
#include <vector>

namespace mpipe
{

class mutable_buffer_basic : public i_mutable_buffer
{
public:
    using stored_data_t = std::vector<std::uint8_t>;
    using u_ptr_t = std::unique_ptr<mutable_buffer_basic>;

    // i_buffer interface
public:
    buffer_type_t buffer_type() const final;
};

class mutable_buffer_ref : public mutable_buffer_basic
{
    stored_data_t*      m_stored_data_ref;
public:

    using u_ptr_t = std::unique_ptr<mutable_buffer_ref>;

    static u_ptr_t create(stored_data_t* stored_data_ref = nullptr);

    mutable_buffer_ref(stored_data_t* stored_data_ref = nullptr);

    virtual void assign(stored_data_t* stored_data_ref);
    // i_data_object interface
public:
    const void *data() const;
    std::size_t size() const;

    // i_buffer interface
public:
    buffer_data_type_t data_type() const;
    std::size_t refs() const;
    i_buffer::u_ptr_t clone() const;

    // i_mapped_buffer interface
public:
    void *map();

    // i_mutable_buffer interface
public:
    std::size_t resize(std::size_t new_size);
    std::size_t append(const i_data_object &data);
};

class mutable_buffer_store : public mutable_buffer_ref
{
    stored_data_t       m_stored_data;

public:
    using u_ptr_t = std::unique_ptr<mutable_buffer_store>;

    static u_ptr_t create(stored_data_t&& stored_data = {});

    mutable_buffer_store(stored_data_t&& stored_data = {});

    virtual void assign(stored_data_t* stored_data_ref) final;
    void assign(stored_data_t&& stored_data_ref);
    // i_data_object interface

    // i_buffer interface
public:
    buffer_data_type_t data_type() const override;
};

}

#endif // MPIPE_MUTABLE_BUFFER_IMPL_H
