#ifndef MPIPE_AUDIO_FORMAT_IMPL_H
#define MPIPE_AUDIO_FORMAT_IMPL_H

#include "i_audio_format.h"
#include "audio_info.h"
#include "media_params.h"
#include <string>

namespace mpipe
{

class audio_format_impl : public i_audio_format
{
    audio_info_t            m_audio_info;
    media_params_t          m_params;

public:
    using u_ptr_t = std::unique_ptr<audio_format_impl>;
    using s_ptr_t = std::shared_ptr<audio_format_impl>;

    static u_ptr_t create(const audio_format_id_t& format_id = audio_format_id_t::undefined
                          , std::int32_t sample_rate = 0
                          , std::int32_t channels = 0
                          , const media_params_t& params = {});

    static u_ptr_t create(const audio_info_t& info
                          , const media_params_t& params = {});

    static u_ptr_t create(const i_audio_format& other);

    audio_format_impl(const audio_format_id_t& format_id = audio_format_id_t::undefined
                      , std::int32_t sample_rate = 0
                      , std::int32_t channels = 0
                     , const media_params_t& params = {});

    audio_format_impl(const audio_info_t& info
                 , const media_params_t& params = {});

    audio_format_impl(const i_audio_format& other);

    const audio_info_t& info() const;
    audio_info_t& info();
    media_params_t& params();

    audio_format_impl& set_format_id(const audio_format_id_t& format_id);
    audio_format_impl& set_sample_rate(std::int32_t sample_rate);
    audio_format_impl& set_channels(std::int32_t channels);
    audio_format_impl& set_params(const media_params_t& params);
    audio_format_impl& set_params(media_params_t&& params);
    audio_format_impl& set_audio_info(const audio_info_t& info);

    audio_format_impl& assign(const i_audio_format& other);

    // i_media_format interface
public:
    media_type_t media_type() const override;
    bool is_encoded() const override;
    bool is_convertable() const override;
    i_media_format::u_ptr_t clone() const override;
    bool is_equal(const i_media_format &other) const override;
    bool is_compatible(const i_media_format &other) const override;
    const media_params_t& params() const override;
    bool is_valid() const override;

    // i_audio_format interface
public:
    audio_format_id_t format_id() const override;
    int32_t sample_rate() const override;
    int32_t channels() const override;
};

}

#endif // MPIPE_AUDIO_FORMAT_IMPL_H
