#ifndef MPIPE_AUDIO_FRAME_IMPL_H
#define MPIPE_AUDIO_FRAME_IMPL_H

#include "frame_info.h"
#include "audio_format_impl.h"
#include "i_audio_frame.h"

namespace mpipe
{

class audio_frame_impl_basic : public i_audio_frame
{
protected:
    audio_format_impl       m_format;
    frame_info_t            m_info;
public:

    audio_frame_impl_basic(const i_audio_format& format
                           , const frame_info_t& info);
    audio_frame_impl_basic(audio_format_impl&& format
                           , frame_info_t&& info = {});

    void set_format(const i_audio_format& format);
    void set_format(audio_format_impl&& format);
    void set_info(const frame_info_t& info);
    void set_info(frame_info_t&& info);

    frame_info_t& info();
    const frame_info_t& info() const;
    audio_format_impl& format();

    std::size_t frame_size() const;

    // i_media_frame interface
public:
    const i_audio_format& format() const final override;

    // i_media_frame interface
public:
    media_type_t media_type() const final override;

    // i_media_frame interface
public:
    frame_id_t frame_id() const override;
    timestamp_t timestamp() const override;
    timestamp_t abs_timestamp() const override;
};

template<typename Buffer>
class audio_frame_impl : public audio_frame_impl_basic
{
    Buffer      m_frame_buffer;

public:

    using u_ptr_t = std::unique_ptr<audio_frame_impl<Buffer>>;
    using s_ptr_t = std::shared_ptr<audio_frame_impl<Buffer>>;

    static u_ptr_t create(const i_audio_format& format
                          , const frame_info_t& info = {}
                          , const Buffer& buffer = {});
    static u_ptr_t create(audio_format_impl&& format = {}
                          , frame_info_t&& info = {}
                          , Buffer&& buffer = {});

    audio_frame_impl(const i_audio_format& format
                , const frame_info_t& info = {}
                , const Buffer& buffer = {});
    audio_frame_impl(audio_format_impl&& format = {}
                , frame_info_t&& info = {}
                , Buffer&& buffer = {});

    void set_frame_data(const Buffer& buffer);
    void set_frame_data(Buffer&& buffer);

    Buffer& frame_buffer();

    // i_media_frame interface
public:
    const i_buffer &frame_data() const override;

    // i_media_frame interface
    i_media_frame::u_ptr_t clone() const override;
};

}

#endif // MPIPE_AUDIO_FRAME_IMPL_H
