#ifndef WBS_MEDIA_AUDIO_INFO_H
#define WBS_MEDIA_AUDIO_INFO_H

#include "core/time/time_types.h"
#include "media_info.h"
#include "audio_types.h"
#include <string>

namespace mpipe
{

class i_audio_format;

struct audio_info_t : public media_info_t
{
    struct format_info_t
    {
        static const format_info_t& get_info(audio_format_id_t format_id);
        static audio_format_id_t format_from_fourcc(std::uint32_t fourcc);

        std::int32_t    bps;
        std::int32_t    frame_size; // in samples
        bool            encoded;
        bool            planar;
        bool            convertable;
        std::uint32_t   fourcc;
    };

    audio_format_id_t   format_id;
    std::uint32_t       sample_rate;
    std::uint32_t       channels;

    audio_info_t(audio_format_id_t format_id = audio_format_id_t::undefined
                  , std::uint32_t sample_rate = 0
                  , std::uint32_t channels = 0);

    audio_info_t(const i_audio_format& audio_format);

    bool operator == (const audio_info_t& other) const;
    bool operator != (const audio_info_t& other) const;

    std::size_t bps() const;
    std::size_t sample_size() const;

    std::size_t size_from_samples(std::size_t samples) const;
    std::size_t size_from_duration(timestamp_t duration) const;

    std::size_t samples_from_size(std::size_t size) const;
    std::size_t samples_from_duration(timestamp_t duration) const;

    timestamp_t duration_from_size(std::size_t size) const;
    timestamp_t duration_from_samples(std::size_t samples) const;

    bool is_valid() const;
    bool is_compatible(const audio_info_t& other) const;
    bool is_convertable() const;
    bool is_encoded() const;
    bool is_planar() const;

    std::string to_string() const override;
};

}

#endif // WBS_MEDIA_AUDIO_INFO_H
