#ifndef MPIPE_I_MEDIA_FRAME_H
#define MPIPE_I_MEDIA_FRAME_H

#include "media_types.h"
#include "frame_types.h"
#include "core/time/time_types.h"
#include <memory>

namespace mpipe
{

class i_buffer;

class i_media_frame
{
public:
    using u_ptr_t = std::unique_ptr<i_media_frame>;
    using s_ptr_t = std::shared_ptr<i_media_frame>;

    virtual ~i_media_frame() = default;
    virtual media_type_t media_type() const = 0;

    virtual frame_id_t frame_id() const = 0;
    virtual timestamp_t timestamp() const = 0;
    virtual timestamp_t abs_timestamp() const = 0;
    virtual const i_buffer& frame_data() const = 0;
    virtual u_ptr_t clone() const = 0;
};

template<class Format>
class i_template_media_frame : public i_media_frame
{
public:
    using u_ptr_t = std::unique_ptr<i_template_media_frame<Format>>;
    using s_ptr_t = std::shared_ptr<i_template_media_frame<Format>>;

    virtual const Format& format() const = 0;
};

}

#endif // MPIPE_I_MEDIA_FRAME_H
