#ifndef MPIPE_MEDIA_PARAMS_H
#define MPIPE_MEDIA_PARAMS_H

#include "core/array_types.h"
#include <cstdint>
#include <string>
#include <vector>

namespace mpipe
{

struct media_params_t
{
    using extra_data_t = octet_array_t;

    std::string     name;
    std::size_t     bitrate;
    std::size_t     gop;
    std::size_t     frame_size;
    std::uint32_t   profile_id;
    std::uint32_t   level_id;
    extra_data_t    extra_data;
    std::string     options;
    std::uint32_t   tag;

    media_params_t(const std::string_view& name = {}
                   , std::size_t bitrate = {}
                   , std::size_t gop = {}
                   , std::size_t frame_size = {}
                   , std::uint32_t profile_id = {}
                   , std::uint32_t level_id = {}
                   , const extra_data_t& extra_data = {}
                   , const std::string_view& options = {}
                   , std::uint32_t tag = 0);

    bool operator == (const media_params_t& other) const;
    bool operator != (const media_params_t& other) const;

    void merge(const media_params_t& other);

    bool is_defined() const;

    void reset();
};

}

#endif // MPIPE_MEDIA_PARAMS_H
