#ifndef MPIPE_VIDEO_FORMAT_IMPL_H
#define MPIPE_VIDEO_FORMAT_IMPL_H

#include "i_video_format.h"
#include "video_info.h"
#include "media_params.h"
#include <string>

namespace mpipe
{

class video_format_impl : public i_video_format
{
    video_info_t            m_video_info;
    media_params_t          m_params;

public:
    using u_ptr_t = std::unique_ptr<video_format_impl>;
    using s_ptr_t = std::shared_ptr<video_format_impl>;

    static video_format_impl& undefined_video_fromat();

    static u_ptr_t create(const video_format_id_t& format_id = video_format_id_t::undefined
                          , std::int32_t width = 0
                          , std::int32_t height = 0
                          , double frame_rate = 0.0
                          , const media_params_t& params = {});

    static u_ptr_t create(const video_info_t& video_info
                          , const media_params_t& params = {});


    static u_ptr_t create(const i_video_format& other);

    video_format_impl(const video_format_id_t& format_id = video_format_id_t::undefined
                      , std::int32_t width = 0
                      , std::int32_t height = 0
                      , double frame_rate = 0.0
                      , const media_params_t& params = {});

    video_format_impl(const video_info_t& video_info
                 , const media_params_t& params = {});

    video_format_impl(const i_video_format& other);

    const video_info_t& info() const;
    video_info_t& info();
    media_params_t& params();

    video_format_impl& set_format_id(const video_format_id_t& format_id);
    video_format_impl& set_width(std::int32_t width);
    video_format_impl& set_height(std::int32_t height);
    video_format_impl& set_frame_rate(double frame_rate);
    video_format_impl& set_params(media_params_t&& params);
    video_format_impl& set_params(const media_params_t& params);
    video_format_impl& set_video_info(const video_info_t& info);
    video_format_impl& assign(const i_video_format& other);

    // i_media_format interface
public:
    media_type_t media_type() const override;
    bool is_encoded() const override;
    bool is_convertable() const override;
    i_media_format::u_ptr_t clone() const override;
    bool is_equal(const i_media_format &other) const override;
    bool is_compatible(const i_media_format &other) const override;
    const media_params_t& params() const override;
    bool is_valid() const override;

    // i_video_format interface
public:
    video_format_id_t format_id() const override;
    int32_t width() const override;
    int32_t height() const override;
    double frame_rate() const override;
};

}

#endif // MPIPE_VIDEO_FORMAT_IMPL_H
