#ifndef MPIPE_VIDEO_FRAME_IMPL_H
#define MPIPE_VIDEO_FRAME_IMPL_H

#include "frame_info.h"
#include "video_format_impl.h"
#include "i_video_frame.h"

namespace mpipe
{

class video_frame_impl_basic : public i_video_frame
{
protected:
    video_format_impl        m_format;
    frame_info_t        m_info;
public:

    video_frame_impl_basic(const i_video_format& format
                           , const frame_info_t& info = {});
    video_frame_impl_basic(video_format_impl&& format
                           , frame_info_t&& info);

    void set_format(const i_video_format& format);
    void set_format(video_format_impl&& format);
    void set_info(const frame_info_t& info);
    void set_info(frame_info_t&& info);

    frame_info_t& info();
    const frame_info_t& info() const;
    video_format_impl& format();
    // i_media_frame interface
public:
    const i_video_format& format() const final override;

    // i_media_frame interface
public:
    media_type_t media_type() const final override;

    // i_media_frame interface
public:
    frame_id_t frame_id() const override;
    timestamp_t timestamp() const override;
    timestamp_t abs_timestamp() const override;

    // i_video_frame interface
    virtual bool key_frame() const;
};

template<class Buffer>
class video_frame_impl : public video_frame_impl_basic
{
    Buffer      m_frame_buffer;

public:

    using u_ptr_t = std::unique_ptr<video_frame_impl<Buffer>>;
    using s_ptr_t = std::shared_ptr<video_frame_impl<Buffer>>;

    static u_ptr_t create(const i_video_format& format
                          , const frame_info_t& info = {}
                          , const Buffer& buffer = {});
    static u_ptr_t create(video_format_impl&& format = {}
                          , frame_info_t&& info = {}
                          , Buffer&& buffer = {});

    video_frame_impl(const i_video_format& format
                , const frame_info_t& info = {}
                , const Buffer& buffer= {});
    video_frame_impl(video_format_impl&& format = {}
                , frame_info_t&& info = {}
                , Buffer&& buffer = {});

    void set_frame_data(const Buffer& buffer);
    void set_frame_data(Buffer&& buffer);

    Buffer& frame_buffer();

    // i_media_frame interface
public:
    const i_buffer &frame_data() const override;

    // i_media_frame interface
    i_media_frame::u_ptr_t clone() const override;
};


}

#endif // MPIPE_VIDEO_FRAME_IMPL_H
