#ifndef MPIPE_MEDIA_VIDEO_INFO_H
#define MPIPE_MEDIA_VIDEO_INFO_H

#include "tools/utils/frame_base.h"
#include "media_info.h"
#include "video_types.h"

#include <string>

namespace mpipe
{

class i_video_format;

struct video_info_t : public media_info_t
{
    using frame_point_t = pt::utils::frame_point_t;
    using frame_size_t = pt::utils::frame_size_t;
    using frame_rect_t = pt::utils::frame_rect_t;

    static constexpr std::uint32_t sample_rate = 90000;

    struct format_info_t
    {
        static const format_info_t& get_info(video_format_id_t format_id);
        static video_format_id_t format_from_fourcc(std::uint32_t fourcc);


        std::int32_t    bpp;
        bool            encoded;
        bool            planar;
        bool            convertable;
        bool            motion;
        std::uint32_t   channels;
        std::int32_t    align_widht;
        std::int32_t    align_height;
        std::uint32_t   fourcc;
    };

    video_format_id_t   format_id;
    frame_size_t        size;
    double              frame_rate;

    video_info_t(video_format_id_t format_id = video_format_id_t::undefined
                 , const frame_size_t& size = {}
                 , double frame_rate = 0.0);

    video_info_t(const i_video_format& video_format);

    bool operator == (const video_info_t& other) const;
    bool operator != (const video_info_t& other) const;

    std::size_t bpp() const;
    std::size_t frame_size() const;
    std::uint32_t frame_time() const;

    bool is_valid() const;
    bool is_compatible(const video_info_t& other) const;
    bool is_convertable() const;
    bool is_encoded() const;
    bool is_planar() const;
    bool is_motion() const;

    std::string to_string() const override;
};

}

#endif // MPIPE_MEDIA_VIDEO_INFO_H
