#ifndef MPIPE_WRTC_I_PEER_CONNECTION_H
#define MPIPE_WRTC_I_PEER_CONNECTION_H

#include "core/buffer/i_data_object.h"
#include "rtc_types.h"
#include "sdp_description.h"
#include <memory>

namespace mpipe::wrtc
{

struct sdp_description_t;
class i_rtc_session;
struct peer_bitrate_params_t;

class i_peer_connection
{
public:

    class i_listener
    {

    public:

        virtual ~i_listener() = default;
        virtual void on_ice_state(ice_connection_state_t ice_state) = 0;
        virtual void on_connection_state(connection_state_t connection_state) = 0;
        virtual void on_gathering_state(ice_gathering_state_t gathering_state) = 0;
        virtual void on_signalling_state(signalling_state_t signalling_state) = 0;
        virtual void on_data_channel_state(data_channel_state_t data_channel_state) = 0;

        virtual void on_set_sdp(const sdp_description_t& sdp_description
                                , peer_type_t peer_type
                                , bool success) = 0;
        virtual void on_create_sdp(const sdp_description_t& sdp_description) = 0;
        virtual void on_session(i_rtc_session* rtc_session) = 0;
        virtual void on_data(const i_data_object& data) = 0;
    };

    using u_ptr_t = std::unique_ptr<i_peer_connection>;
    using s_ptr_t = std::shared_ptr<i_peer_connection>;

    virtual ~i_peer_connection() = default;

    virtual std::size_t id() const = 0;
    virtual ice_connection_state_t ice_connection_state() const = 0;
    virtual connection_state_t connection_state() const = 0;
    virtual ice_gathering_state_t ice_gathering_state() const = 0;
    virtual signalling_state_t signalling_state() const = 0;
    virtual data_channel_state_t data_channel_state() const = 0;

    virtual void set_listener(i_listener* peer_listener) = 0;
    virtual i_rtc_session* get_session(std::size_t session_id) = 0;
    virtual bool send_data(const i_data_object& data) = 0;

    virtual bool set_bitrate_params(const peer_bitrate_params_t& bitrate_params) = 0;
    virtual bool get_bitrate_params(peer_bitrate_params_t& bitrate_params) const = 0;

    virtual bool create_sdp(sdp_type_t sdp_type) = 0;
    virtual bool set_sdp(peer_type_t peer_type
                         , const sdp_description_t& sdp_description) = 0;
    virtual sdp_description_t get_sdp(peer_type_t peer_type) const = 0;

};

}

#endif // MPIPE_WRTC_I_PEER_CONNECTION_H
