#ifndef MPIPE_RTC_TRACK_PARAMS_H
#define MPIPE_RTC_TRACK_PARAMS_H

#include "rtc_types.h"
#include "core/media/media_types.h"

#include <string>
#include <vector>

namespace mpipe::wrtc
{

struct track_params_t
{
    struct bitrate_params_t
    {
        std::uint32_t       min_bitrate;
        std::uint32_t       max_bitrate;

        static const bitrate_params_t& undefined();

        bitrate_params_t(std::uint32_t min_bitrate = 0
                         , std::uint32_t max_bitrate = 0);

        bool operator == (const bitrate_params_t& other) const;
        bool operator != (const bitrate_params_t& other) const;

        bool is_defined() const;
    };

    using array_t = std::vector<track_params_t>;

    media_type_t                media_type;
    rtc_track_direction_t       direction;
    std::string                 track_id;
    bitrate_params_t            bitrate;

    track_params_t(media_type_t media_type = media_type_t::undefined
                    , rtc_track_direction_t direction = rtc_track_direction_t::undefined
                    , const std::string_view& track_id = {}
                    , const bitrate_params_t& bitrate = {});


    bool is_valid() const;

    bool has_recv() const;
    bool has_send() const;
};

}

#endif // MPIPE_RTC_TRACK_PARAMS_H
