#ifndef MPIPE_UTILS_DEVICE_LISTENER_IMPL_H
#define MPIPE_UTILS_DEVICE_LISTENER_IMPL_H

#include "core/device/i_device.h"
#include <functional>

namespace mpipe::utils::device
{

class device_listener_impl : public i_device::i_listener
{
    using stream_handler_t = std::function<void(i_media_stream *stream
                                                , stream_state_t event)>;
    using state_handler_t = std::function<void(device_state_t state)>;
    using command_handler_t = std::function<bool(const i_command &command)>;

    stream_handler_t        m_stream_handler;
    state_handler_t         m_state_handler;
    command_handler_t       m_command_handler;

public:

    device_listener_impl(stream_handler_t stream_handler = nullptr
                         , state_handler_t state_handler = nullptr
                         , command_handler_t command_handler = nullptr);

    void set_stream_handler(stream_handler_t stream_handler);
    void set_state_handler(state_handler_t state_handler);
    void set_command_handler(command_handler_t command_handler);

    // i_listener interface
public:
    void on_media_stream(i_media_stream *stream
                         , stream_state_t state) override;

    // i_listener interface
public:
    void on_device_state(device_state_t state) override;
    bool on_device_command(const i_command &command) override;
};

}

#endif // MPIPE_UTILS_DEVICE_LISTENER_IMPL_H
