#ifndef MPIPE_MEDIA_ROUTER_IMPL_H
#define MPIPE_MEDIA_ROUTER_IMPL_H

#include "core/media/i_media_router.h"
#include "tools/utils/sync_base.h"
#include <unordered_set>

namespace mpipe::utils::media
{

class media_router_impl : public i_media_router
{
    using mutex_t = pt::utils::shared_spin_lock;
    using sink_set_t = std::unordered_set<i_media_sink*>;

    mutable mutex_t         m_sync_mutex;
    sink_set_t              m_sinks;

public:

    using u_ptr_t = std::unique_ptr<media_router_impl>;

    static u_ptr_t create(i_media_sink *sink = nullptr);

    media_router_impl(i_media_sink *sink = nullptr);

    std::size_t sinks() const;

    // i_media_source interface
public:
    bool add_sink(i_media_sink *sink) override;
    bool remove_sink(i_media_sink *sink) override;

    // i_media_sink interface
public:
    bool send_frame(const i_media_frame &frame) override;
    bool send_command(const i_command &command) override;
};

}

#endif // MPIPE_MEDIA_ROUTER_IMPL_H
