#ifndef MPIPE_MEDIA_SINK_IMPL_H
#define MPIPE_MEDIA_SINK_IMPL_H

#include "core/media/i_media_sink.h"
#include <functional>

namespace mpipe::utils::media
{

class media_sink_impl : public i_media_sink
{
    using frame_handler_t = std::function<bool(const i_media_frame&)>;
    using command_handler_t = std::function<bool(const i_command&)>;

    frame_handler_t     m_frame_handler;
    command_handler_t   m_command_handler;

public:

    using u_ptr_t = std::unique_ptr<media_sink_impl>;

    static u_ptr_t create(frame_handler_t&& frame_handler = {}
                          , command_handler_t&& command_handler = {});

    media_sink_impl(frame_handler_t&& frame_handler = {}
                    , command_handler_t&& command_handler = {});

    void set_frame_handler(frame_handler_t&& frame_handler);
    void set_command_handler(command_handler_t&& command_handler);

    // i_media_sink interface
public:
    bool send_frame(const i_media_frame &frame) override;
    bool send_command(const i_command &command) override;
};

}

#endif // MPIPE_MEDIA_SINK_IMPL_H
