#ifndef MPIPE_OBJECT_COUNTER_H
#define MPIPE_OBJECT_COUNTER_H

#include <atomic>

namespace mpipe::utils
{

template<typename Object, typename CounterType = std::size_t>
class object_counter
{
private:
    static CounterType next_id()
    {
        static std::atomic<CounterType> single_counter = 0;
        return single_counter.fetch_add(1, std::memory_order_relaxed);
    }

protected:
    CounterType         m_object_id;

    object_counter()
        : m_object_id(next_id())
    {

    }

public:
    inline CounterType counted_object_id() const { return m_object_id; }
};

}

#endif // MPIPE_OBJECT_COUNTER_H
