#ifndef MPIPE_UTILS_OPTION_READER_H
#define MPIPE_UTILS_OPTION_READER_H

#include "core/option/i_option.h"
#include "option_utils.h"
#include <optional>

namespace mpipe::utils::option
{

class option_reader
{
protected:
    const i_option&     m_root;
public:
    option_reader(const i_option& root);

    const i_option* get_child(const std::string& key) const;

    template<typename T>
    inline bool get(const std::string& key, T& value) const
    {
        if (auto child = get_child(key))
        {
            return deserialize(*child
                               , value);
        }

        return false;
    }

    template<typename T>
    inline T get(const std::string& key, const T& default_value) const
    {
        T result = default_value;
        get(key, result);
        return result;
    }

    template<typename T>
    inline std::optional<T> get_optional(const std::string& key) const
    {
        T result = {};
        if (get(key, result))
        {
            return result;
        }

        return std::nullopt;
    }
};

}

#endif // MPIPE_UTILS_OPTION_READER_H
